﻿using System.Diagnostics;

namespace OBSBlur.Window;

public struct WindowInfo
{
    internal IntPtr WindowHandle { get; init; }
    public bool IsVisible { get; init; }
    public ShowWindowCommands WindowCommands { get; init; }
    public Rectangle WindowRectangle { get; init; }
    public string WindowTitle { get; init; }
    public Process ProcessInfo { get; init; }

    public WindowInfo(IntPtr windowHandle, string windowTitle, Process processInfo, bool isWindowVisible, ShowWindowCommands windowCommands, Rectangle windowRectangle)
    {
        this.WindowHandle = windowHandle;
        this.ProcessInfo = processInfo;
        this.IsVisible = isWindowVisible;
        this.WindowCommands = windowCommands;
        this.WindowRectangle = windowRectangle;
        this.WindowTitle = windowTitle;
    }

    public override bool Equals(object? obj)
    {
        return obj is WindowInfo other && Equals(other);
    }

    public bool Equals(WindowInfo other)
    {
        return WindowHandle == other.WindowHandle &&
               IsVisible == other.IsVisible &&
               WindowCommands == other.WindowCommands &&
               WindowRectangle == other.WindowRectangle &&
               WindowTitle.Equals(other.WindowTitle) &&
               ProcessInfo.ProcessName.Equals(other.ProcessInfo.ProcessName) &&
               ProcessInfo.Id == other.ProcessInfo.Id;
    }

    public override int GetHashCode()
    {
        return HashCode.Combine(WindowHandle, IsVisible, WindowTitle, ProcessInfo);
    }

    public override string ToString()
    {
        const int cutoffStr = 17;
        string processNameStr = ProcessInfo.ProcessName.Substring(0, Math.Min(cutoffStr, ProcessInfo.ProcessName.Length));
        string windowTitleStr = WindowTitle.Substring(0, Math.Min(cutoffStr, WindowTitle.Length));
        return $"{WindowHandle,8} | {ProcessInfo.Id,8} | {processNameStr,cutoffStr} | {windowTitleStr,-cutoffStr} | {(IsVisible ? "visible" : "hidden"),-8} | {WindowCommands,-13} | {WindowRectangle}";
    }
}